"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LinesAndColumns = exports.Context = void 0;
const lodash_1 = __importDefault(require("lodash"));
const template_scope_manager_1 = require("./template-scope-manager");
class Context {
    constructor(code, parserOptions) {
        this.tokens = [];
        this.comments = [];
        this.locsMap = new Map();
        this._templateScopeManager = null;
        this.code = code;
        this.parserOptions = parserOptions;
        this.locs = new LinesAndColumns(code);
        const sourceCode = {
            svelte: code,
            script: {
                code: code.replace(/[^\n\r ]/g, " "),
                ranges: [],
                attrs: {},
            },
        };
        for (const script of extractScriptBlocks(code)) {
            sourceCode.svelte =
                sourceCode.svelte.slice(0, script.codeRange[0]) +
                    script.code.replace(/[^\n\r ]/g, " ") +
                    sourceCode.svelte.slice(script.codeRange[1]);
            sourceCode.script.code =
                sourceCode.script.code.slice(0, script.codeRange[0]) +
                    script.code +
                    sourceCode.script.code.slice(script.codeRange[1]);
            sourceCode.script.ranges.push({
                code: script.codeRange,
                tag: script.tagRange,
            });
            sourceCode.script.attrs = Object.assign(sourceCode.script.attrs, script.attrs);
        }
        this.sourceCode = sourceCode;
    }
    getLocFromIndex(index) {
        let loc = this.locsMap.get(index);
        if (!loc) {
            loc = this.locs.getLocFromIndex(index);
            this.locsMap.set(index, loc);
        }
        return {
            line: loc.line,
            column: loc.column,
        };
    }
    /**
     * Get the location information of the given node.
     * @param node The node.
     */
    getConvertLocation(node) {
        const { start, end } = node;
        return {
            range: [start, end],
            loc: {
                start: this.getLocFromIndex(start),
                end: this.getLocFromIndex(end),
            },
        };
    }
    addComment(comment) {
        this.comments.push(comment);
    }
    /**
     * Add token to tokens
     */
    addToken(type, range) {
        const token = Object.assign({ type, value: this.getText(range) }, this.getConvertLocation(range));
        this.tokens.push(token);
        return token;
    }
    /**
     * get text
     */
    getText(range) {
        return this.code.slice(range.start, range.end);
    }
    readyScopeManager(scopeManager) {
        this._templateScopeManager = new template_scope_manager_1.TemplateScopeManager(scopeManager);
    }
    get templateScopeManager() {
        return this._templateScopeManager;
    }
}
exports.Context = Context;
/** Extract <script> blocks */
function* extractScriptBlocks(code) {
    const scriptRe = /<script(\s[\s\S]*?)?>([\s\S]*?)<\/script>/giu;
    let res;
    while ((res = scriptRe.exec(code))) {
        const [tag, attributes = "", context] = res;
        const tagRange = [res.index, scriptRe.lastIndex];
        const codeRange = [
            tagRange[0] + 8 + attributes.length,
            tagRange[1] - 9,
        ];
        // eslint-disable-next-line regexp/no-unused-capturing-group -- maybe bug
        const attrRe = /(<key>[^\s=]+)(?:=(?:"(<val>[^"]*)"|'(<val>[^"]*)'|(<val>[^\s=]+)))?/giu;
        const attrs = {};
        while ((res = attrRe.exec(attributes))) {
            attrs[res.groups.key] = res.groups.val;
        }
        yield {
            code: context,
            codeRange,
            tag,
            tagRange,
            attrs,
        };
    }
}
class LinesAndColumns {
    constructor(code) {
        const len = code.length;
        const lineStartIndices = [0];
        for (let index = 0; index < len; index++) {
            const c = code[index];
            if (c === "\r") {
                const next = code[index + 1] || "";
                if (next === "\n") {
                    index++;
                }
                lineStartIndices.push(index + 1);
            }
            else if (c === "\n") {
                lineStartIndices.push(index + 1);
            }
        }
        this.lineStartIndices = lineStartIndices;
    }
    getLocFromIndex(index) {
        const lineNumber = lodash_1.default.sortedLastIndex(this.lineStartIndices, index);
        return {
            line: lineNumber,
            column: index - this.lineStartIndices[lineNumber - 1],
        };
    }
    getIndexFromLoc(loc) {
        const lineStartIndex = this.lineStartIndices[loc.line - 1];
        const positionIndex = lineStartIndex + loc.column;
        return positionIndex;
    }
}
exports.LinesAndColumns = LinesAndColumns;
